/* STARTUP(crt0.o) */
/* To generate PIC code use only elf32-m68k and then convert to srec */
OUTPUT_FORMAT(elf32-m68k)
OUTPUT_ARCH(m68k)
SEARCH_DIR(.)
__DYNAMIC  =  0;

/*
 * Setup the memory map of the Khepera
 *
 * The memory map look like this:
 * +--------------------+ <- low memory
 * | .text              |
 * |        _etext      |
 * |        ctor list   | the ctor and dtor lists are for
 * |        dtor list   | C++ support
 * +--------------------+
 * | .data              | initialized data goes here
 * |        _edata      |
 * +--------------------+
 * | .bss               |
 * |        __bss_start | start of bss, cleared by crt0
 * |        _end        | start of heap, used by sbrk()
 * +--------------------+
 * .                    .
 * .                    .
 * .                    .
 * |        __stack     | top of stack
 * +--------------------+
 */
MEMORY
{
  rom     : ORIGIN = 0x000000, LENGTH = 128K
  ramsys  : ORIGIN = 0x120000, LENGTH = 32K
  ramcode : ORIGIN = 0x128000, LENGTH = 96K
  ramdata : ORIGIN = 0x1A0000, LENGTH = 128K
}

/*
 * allocate the stack to be at the top of memory, since the stack
 * grows down
 */
PROVIDE (__stack = 0x140000);

PROVIDE (ApplStr = 0);
PROVIDE (HelpStr = 0);

/*
 * Initalize some symbols to be zero so we can reference them in the
 * crt0 without core dumping. These functions are all optional, but
 * we do this so we can have our crt0 always use them if they exist. 
 * This is so BSPs work better when using the crt0 installed with gcc.
 */
PROVIDE (hardware_init_hook = 0);
PROVIDE (_hardware_init_hook = 0);
PROVIDE (software_init_hook = 0);
PROVIDE (_software_init_hook = 0);

/*
 * Module identity
 * 	"FU" stand alone executable function code
 * 	"PR" SerCom protocol function code
 * 	"TO" SerCom tool function code
 * 	"BI" Bios
 * 	"ST" Start-up code
 * 	"LI" Lib
 * 	"DG" Debug
 * 	"TA" Proc
 */
PROVIDE (idmodule = 0x46555541);

/*
 * File type :
 * 	0 visible in help
 * 	1 executable from SerCom
 * 	2 executable from run
 * 	3 confidential
 */
PROVIDE (fltype = 1);

/*
 * Output type :
 *      0 serial output
 *      1 communication output
 */
PROVIDE (io = 1);

/*
 * stick everything in ram (of course)
 */
SECTIONS
{

  /* Needed without PIC don't work properly */
  .interp     : { *(.interp)    }
  .hash          : { *(.hash)           }
  .dynsym        : { *(.dynsym)         }
  .dynstr        : { *(.dynstr)         }
  .gnu.version   : { *(.gnu.version)    }
  .gnu.version_d   : { *(.gnu.version_d)        }
  .gnu.version_r   : { *(.gnu.version_r)        }
  .rel.text      :
    { *(.rel.text) *(.rel.gnu.linkonce.t*) }
  .rela.text     :
    { *(.rela.text) *(.rela.gnu.linkonce.t*) }
  .rel.data      :
    { *(.rel.data) *(.rel.gnu.linkonce.d*) }
  .rela.data     :
    { *(.rela.data) *(.rela.gnu.linkonce.d*) }
  .rel.rodata    :
    { *(.rel.rodata) *(.rel.gnu.linkonce.r*) }
  .rela.rodata   :
    { *(.rela.rodata) *(.rela.gnu.linkonce.r*) }
  .rel.got       : { *(.rel.got)                }
  .rela.got      : { *(.rela.got)               }
  .rel.ctors     : { *(.rel.ctors)      }
  .rela.ctors    : { *(.rela.ctors)     }
  .rel.dtors     : { *(.rel.dtors)      }
  .rela.dtors    : { *(.rela.dtors)     }
  .rel.init      : { *(.rel.init)       }
  .rela.init     : { *(.rela.init)      }
  .rel.fini      : { *(.rel.fini)       }
  .rela.fini     : { *(.rela.fini)      }
  .rel.bss       : { *(.rel.bss)                }
  .rela.bss      : { *(.rela.bss)               }
  .rel.plt       : { *(.rel.plt)                }
  .rela.plt      : { *(.rela.plt)               }
  .init          : { *(.init)   } 
  .plt      : { *(.plt) }

  .rela.got      : 
  { 
   *(.rela.got)
  } > ramdata

  _start = .;
  /* Code */
  .text :
  {
    *(.text)
    *(.stub)
    *(.gnu.warning)
    *(.gnu.linkonce.t*)
  } > ramdata

  /* Read Only initialized data */
  .rodata BLOCK (0x4) :
  {
    *(.rodata)
    *(.gnu.linkonce.r*)
  } > ramdata

  .gcc_except_table BLOCK (0x4) :
  {
    *(.gcc_except_table) 
  } > ramdata

  /* Writable initialized data */
  .data :
  {
    *(.data)
    *(.gnu.linkonce.d*)      
    CONSTRUCTORS
  } > ramdata

  /* Constructors (C++) */
  .ctors BLOCK (0x4) :
  {
    /* Constructors */
     __CTOR_LIST__ = .;
    *(.ctors)
    __CTOR_END__ = .;
  } > ramdata

  /* Destructors (C++) */
  .dtors BLOCK (0x4) :
  {
    /* Destructors */
    __DTOR_LIST__ = .;
    *(.dtors)
    __DTOR_END__ = .;
  } > ramdata

  /* Global offset table for position independent code generation */
  .got BLOCK(0x4) :
  {
    *(.got.plt)
    *(.got)
  } > ramdata

  /* Writable Uninitialized data */
  .bss BLOCK (0x4) :
  {
    __bss_start = . ;
    *(.dynbss)
    *(.bss)
    *(COMMON)
    __bss_end = . ;
  } > ramdata
  _end =  ALIGN (0x8);

  /* Computation of symbols used for the KTEAM Header */
  ptrrom = ADDR(.text);
  lnrom = _end - _start;
  ptrram = 0x128000;
  /* Also used in sbrk (i.e. malloc) */
  lnram = __stack - 1K;

}
